@@ -54,16 +54,18 @@ enum {
 };
 
 enum {
-  SIGNAL_PROBE_DONE,
-  SIGNAL_SCANNER_SELECTED,
-  SIGNAL_ACQUISITION_STARTED,
-  SIGNAL_DATA_RECEIVED,
-  SIGNAL_ACQUISITION_TERMINATED,
-  SIGNAL_IMAGE_RECEIVED,
-  SIGNAL_PREVIEW_STARTED,
-  SIGNAL_PREVIEW_TERMINATED,
-  SIGNAL_PREVIEW_RECEIVED,
-  SIGNAL_ERROR,
+  PROBE_DONE,
+  SCANNER_SELECTED,
+  ACQUISITION_STARTED,
+  DATA_RECEIVED,
+  ACQUISITION_PAUSED,
+  ACQUISITION_RESUMED,
+  ACQUISITION_TERMINATED,
+  IMAGE_RECEIVED,
+  PREVIEW_STARTED,
+  PREVIEW_TERMINATED,
+  PREVIEW_RECEIVED,
+  ERROR,
   N_SIGNALS
 };
 
@@ -72,12 +74,16 @@ static guint signals[N_SIGNALS];
 void				gsc_trigger_error				(GnomeScanContext *context,
 										 GError *error);
 
-void				gsc_acquisition_threaded			(GnomeScanContext *context);
+void				gsc_acquire					(GnomeScanContext *context);
 
 void				gsc_data_received				(GnomeScanBackend *backend,
 										 gsize data_size,
 										 GnomeScanContext *context);
 
+void				gsc_image_acquired				(GnomeScanBackend *backend,
+										 GnomeScanResult *result,
+										 GnomeScanContext *context);
+
 /********************************
  * 	      GOBJECT		*
  ********************************/
@@ -172,15 +178,15 @@ gnome_scan_context_class_init (GnomeScanContextClass* klass)
    *
    * Emitted when the backend terminate the scanner probe.
    */
-  signals[SIGNAL_PROBE_DONE] = g_signal_new ("probe-done",
-					     GNOME_TYPE_SCAN_CONTEXT,
-					     G_SIGNAL_RUN_FIRST,
-					     G_STRUCT_OFFSET (GnomeScanContextClass, probe_done),
-					     NULL,
-					     NULL,
-					     g_cclosure_marshal_VOID__VOID,
-					     G_TYPE_NONE,
-					     0);
+  signals[PROBE_DONE] = g_signal_new ("probe-done",
+				      GNOME_TYPE_SCAN_CONTEXT,
+				      G_SIGNAL_RUN_FIRST,
+				      G_STRUCT_OFFSET (GnomeScanContextClass, probe_done),
+				      NULL,
+				      NULL,
+				      g_cclosure_marshal_VOID__VOID,
+				      G_TYPE_NONE,
+				      0);
 
   /**
    * GnomeScanContext::scanner-selected:
@@ -190,16 +196,16 @@ gnome_scan_context_class_init (GnomeScanContextClass* klass)
    * Emitted when the current a scanner is selected (wheter or not a
    * previous scanner were selected).
    */
-  signals[SIGNAL_SCANNER_SELECTED] = g_signal_new ("scanner-selected",
-						   GNOME_TYPE_SCAN_CONTEXT,
-						   G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST,
-						   G_STRUCT_OFFSET (GnomeScanContextClass, scanner_selected),
-						   NULL,
-						   NULL,
-						   g_cclosure_marshal_VOID__OBJECT,
-						   G_TYPE_NONE,
-						   1,
-						   GNOME_TYPE_SCANNER);
+  signals[SCANNER_SELECTED] = g_signal_new ("scanner-selected",
+					    GNOME_TYPE_SCAN_CONTEXT,
+					    G_SIGNAL_RUN_FIRST | G_SIGNAL_RUN_LAST,
+					    G_STRUCT_OFFSET (GnomeScanContextClass, scanner_selected),
+					    NULL,
+					    NULL,
+					    g_cclosure_marshal_VOID__OBJECT,
+					    G_TYPE_NONE,
+					    1,
+					    GNOME_TYPE_SCANNER);
 
   /**
    * GnomeScanContext::preview-started:
@@ -209,16 +215,16 @@ gnome_scan_context_class_init (GnomeScanContextClass* klass)
    * Emitted when a preview acquisition is started.
    * 
    */
-  signals[SIGNAL_PREVIEW_STARTED] = g_signal_new ("preview-started",
-						  GNOME_TYPE_SCAN_CONTEXT,
-						  G_SIGNAL_RUN_FIRST,
-						  G_STRUCT_OFFSET (GnomeScanContextClass, preview_started),
-						  NULL,
-						  NULL,
-						  g_cclosure_marshal_VOID__BOXED,
-						  G_TYPE_NONE,
-						  1,
-						  GNOME_TYPE_SCAN_FORECAST);
+  signals[PREVIEW_STARTED] = g_signal_new ("preview-started",
+					   GNOME_TYPE_SCAN_CONTEXT,
+					   G_SIGNAL_RUN_FIRST,
+					   G_STRUCT_OFFSET (GnomeScanContextClass, preview_started),
+					   NULL,
+					   NULL,
+					   g_cclosure_marshal_VOID__BOXED,
+					   G_TYPE_NONE,
+					   1,
+					   GNOME_TYPE_SCAN_FORECAST);
 
   /**
    * GnomeScanContext::preview-terminated:
@@ -227,15 +233,15 @@ gnome_scan_context_class_init (GnomeScanContextClass* klass)
    * Emitted when a preview acquisition is terminated.
    * 
    */
-  signals[SIGNAL_PREVIEW_TERMINATED] = g_signal_new ("preview-terminated",
-						     GNOME_TYPE_SCAN_CONTEXT,
-						     G_SIGNAL_RUN_FIRST,
-						     G_STRUCT_OFFSET (GnomeScanContextClass, preview_received),
-						     NULL,
-						     NULL,
-						     g_cclosure_marshal_VOID__VOID,
-						     G_TYPE_NONE,
-						     0);
+  signals[PREVIEW_TERMINATED] = g_signal_new ("preview-terminated",
+					      GNOME_TYPE_SCAN_CONTEXT,
+					      G_SIGNAL_RUN_FIRST,
+					      G_STRUCT_OFFSET (GnomeScanContextClass, preview_received),
+					      NULL,
+					      NULL,
+					      g_cclosure_marshal_VOID__VOID,
+					      G_TYPE_NONE,
+					      0);
 
   /**
    * GnomeScanContext::preview-received:
@@ -245,16 +251,16 @@ gnome_scan_context_class_init (GnomeScanContextClass* klass)
    * Emitted when a preview image is received.
    * 
    */
-  signals[SIGNAL_PREVIEW_RECEIVED] = g_signal_new ("preview-received",
-						   GNOME_TYPE_SCAN_CONTEXT,
-						   G_SIGNAL_RUN_FIRST,
-						   G_STRUCT_OFFSET (GnomeScanContextClass, preview_received),
-						   NULL,
-						   NULL,
-						   g_cclosure_marshal_VOID__POINTER,
-						   G_TYPE_NONE,
-						   1,
-						   G_TYPE_POINTER);
+  signals[PREVIEW_RECEIVED] = g_signal_new ("preview-received",
+					    GNOME_TYPE_SCAN_CONTEXT,
+					    G_SIGNAL_RUN_FIRST,
+					    G_STRUCT_OFFSET (GnomeScanContextClass, preview_received),
+					    NULL,
+					    NULL,
+					    g_cclosure_marshal_VOID__BOXED,
+					    G_TYPE_NONE,
+					    1,
+					    GNOME_TYPE_SCAN_RESULT);
 
   /**
    * GnomeScanContext::acquisition-started:
@@ -264,35 +270,67 @@ gnome_scan_context_class_init (GnomeScanContextClass* klass)
    * Emitted when the acquisition is started. Use that signal to show
    * a popup before actualy receiving data.
    */
-  signals[SIGNAL_ACQUISITION_STARTED] = g_signal_new ("acquisition-started",
-						      GNOME_TYPE_SCAN_CONTEXT,
-						      G_SIGNAL_RUN_FIRST,
-						      G_STRUCT_OFFSET (GnomeScanContextClass, acquisition_started),
-						      NULL,
-						      NULL,
-						      g_cclosure_marshal_VOID__BOXED,
-						      G_TYPE_NONE,
-						      1,
-						      GNOME_TYPE_SCAN_FORECAST);
+  signals[ACQUISITION_STARTED] = g_signal_new ("acquisition-started",
+					       GNOME_TYPE_SCAN_CONTEXT,
+					       G_SIGNAL_RUN_FIRST,
+					       G_STRUCT_OFFSET (GnomeScanContextClass, acquisition_started),
+					       NULL,
+					       NULL,
+					       g_cclosure_marshal_VOID__BOXED,
+					       G_TYPE_NONE,
+					       1,
+					       GNOME_TYPE_SCAN_FORECAST);
 
   /**
-   * GnomeScanContext::data-received:2
+   * GnomeScanContext::data-received:
    * @context: The emitting #GnomeScanContext
    * @data_size: The current number of bytes received.
    *
    * Emitted when the backend send data. Either for a final picture
    * acquisition or for a preview refresh.
    */
-  signals[SIGNAL_DATA_RECEIVED] = g_signal_new ("data-received",
-						GNOME_TYPE_SCAN_CONTEXT,
-						G_SIGNAL_RUN_FIRST,
-						G_STRUCT_OFFSET (GnomeScanContextClass, data_received),
-						NULL,
-						NULL,
-						g_cclosure_marshal_VOID__INT,
-						G_TYPE_NONE,
-						1,
-						G_TYPE_INT);
+  signals[DATA_RECEIVED] = g_signal_new ("data-received",
+					 GNOME_TYPE_SCAN_CONTEXT,
+					 G_SIGNAL_RUN_FIRST,
+					 G_STRUCT_OFFSET (GnomeScanContextClass, data_received),
+					 NULL,
+					 NULL,
+					 g_cclosure_marshal_VOID__INT,
+					 G_TYPE_NONE,
+					 1,
+					 G_TYPE_INT);
+
+  /**
+   * GnomeScanContext::acquisition-paused:
+   * @context: The emitting #GnomeScanContext
+   *
+   * Emitted when the acquisition process has been paused.
+   */
+  signals[ACQUISITION_PAUSED] = g_signal_new ("acquisition-paused",
+					      GNOME_TYPE_SCAN_CONTEXT,
+					      G_SIGNAL_RUN_FIRST,
+					      G_STRUCT_OFFSET (GnomeScanContextClass, acquisition_paused),
+					      NULL,
+					      NULL,
+					      g_cclosure_marshal_VOID__VOID,
+					      G_TYPE_NONE,
+					      0);
+
+  /**
+   * GnomeScanContext::acquisition-resumed:
+   * @context: The emitting #GnomeScanContext
+   *
+   * Emitted when the acquisition process has been paused.
+   */
+  signals[ACQUISITION_RESUMED] = g_signal_new ("acquisition-resumed",
+					       GNOME_TYPE_SCAN_CONTEXT,
+					       G_SIGNAL_RUN_FIRST,
+					       G_STRUCT_OFFSET (GnomeScanContextClass, acquisition_resumed),
+					       NULL,
+					       NULL,
+					       g_cclosure_marshal_VOID__VOID,
+					       G_TYPE_NONE,
+					       0);
 
   /**
    * GnomeScanContext::acquisition-terminated:
@@ -301,15 +339,15 @@ gnome_scan_context_class_init (GnomeScanContextClass* klass)
    * Emitted when the backend has no more data to send. This signal is
    * emitted even if the acquisition has been canceled.
    */
-  signals[SIGNAL_ACQUISITION_TERMINATED] = g_signal_new ("acquisition-terminated",
-							 GNOME_TYPE_SCAN_CONTEXT,
-							 G_SIGNAL_RUN_FIRST,
-							 G_STRUCT_OFFSET (GnomeScanContextClass, acquisition_terminated),
-							 NULL,
-							 NULL,
-							 g_cclosure_marshal_VOID__VOID,
-							 G_TYPE_NONE,
-							 0);
+  signals[ACQUISITION_TERMINATED] = g_signal_new ("acquisition-terminated",
+						  GNOME_TYPE_SCAN_CONTEXT,
+						  G_SIGNAL_RUN_FIRST,
+						  G_STRUCT_OFFSET (GnomeScanContextClass, acquisition_terminated),
+						  NULL,
+						  NULL,
+						  g_cclosure_marshal_VOID__VOID,
+						  G_TYPE_NONE,
+						  0);
 
   /**
    * GnomeScanContext::image-received:
@@ -320,16 +358,16 @@ gnome_scan_context_class_init (GnomeScanContextClass* klass)
    * sent with that signal.
    * 
    */
-  signals[SIGNAL_IMAGE_RECEIVED] = g_signal_new ("image-received",
-						 GNOME_TYPE_SCAN_CONTEXT,
-						 G_SIGNAL_RUN_FIRST,
-						 G_STRUCT_OFFSET (GnomeScanContextClass, image_received),
-						 NULL,
-						 NULL,
-						 g_cclosure_marshal_VOID__POINTER,
-						 G_TYPE_NONE,
-						 1,
-						 G_TYPE_POINTER);
+  signals[IMAGE_RECEIVED] = g_signal_new ("image-received",
+					  GNOME_TYPE_SCAN_CONTEXT,
+					  G_SIGNAL_RUN_FIRST,
+					  G_STRUCT_OFFSET (GnomeScanContextClass, image_received),
+					  NULL,
+					  NULL,
+					  g_cclosure_marshal_VOID__BOXED,
+					  G_TYPE_NONE,
+					  1,
+					  GNOME_TYPE_SCAN_RESULT);
 
   /**
    * GnomeScanContext::error:
@@ -339,16 +377,16 @@ gnome_scan_context_class_init (GnomeScanContextClass* klass)
    * This signal allow the application to avert he user that a backend
    * error occured.
    */
-  signals[SIGNAL_ERROR] = g_signal_new ("error",
-					GNOME_TYPE_SCAN_CONTEXT,
-					G_SIGNAL_RUN_FIRST,
-					G_STRUCT_OFFSET (GnomeScanContextClass, error),
-					NULL,
-					NULL,
-					g_cclosure_marshal_VOID__POINTER,
-					G_TYPE_NONE,
-					1,
-					G_TYPE_POINTER);
+  signals[ERROR] = g_signal_new ("error",
+				 GNOME_TYPE_SCAN_CONTEXT,
+				 G_SIGNAL_RUN_FIRST,
+				 G_STRUCT_OFFSET (GnomeScanContextClass, error),
+				 NULL,
+				 NULL,
+				 g_cclosure_marshal_VOID__POINTER,
+				 G_TYPE_NONE,
+				 1,
+				 G_TYPE_POINTER);
 }
 
 void
@@ -493,6 +531,11 @@ gnome_scan_context_new (void)
 		    (GCallback) gsc_data_received,
 		    context);
 
+  g_signal_connect (backend,
+		    "image-acquired",
+		    (GCallback) gsc_image_acquired,
+		    context);
+
   return context;
 }
 
@@ -533,7 +576,7 @@ gnome_scan_context_probe_scanners (GnomeScanContext *context)
   if (error) {
     gsc_trigger_error (context, error);
   }
-
+  
   g_signal_emit_by_name (context,
 			 "probe-done");
 }
@@ -709,10 +752,10 @@ gnome_scan_context_get_rotation (GnomeScanContext *context)
 
 /**
  * gnome_scan_context_set_source:
- * @context: 
- * @source: 
+ * @context:	a #GonmeScanContext
+ * @source: 	the source to select
  * 
- * Retreive the current selected source.
+ * Change current selected source.
  **/
 void
 gnome_scan_context_set_source (GnomeScanContext *context,
@@ -761,6 +804,7 @@ gnome_scan_context_acquire_preview (GnomeScanContext *context)
   priv = GET_PRIVATE (context);
   scanner = priv->info->scanner;
 
+  /* We create a temp ContextInfo suitable for preview. */
   /* resolution */
   info = g_new0 (GnomeScanContextInfo, 1);
   info->resolution = 50;
@@ -776,24 +820,10 @@ gnome_scan_context_acquire_preview (GnomeScanContext *context)
 
   fc =  gnome_scan_backend_forecast (GET_PRIVATE (context)->backend, info, &error);
   if (error) gsc_trigger_error (context, error);
-  g_signal_emit_by_name (context, "preview-started", fc);
-  while (gtk_events_pending ())  gtk_main_iteration ();
 
-  result = gnome_scan_backend_acquire (priv->backend, info, &error);
-  g_signal_emit_by_name (context, "preview-terminated");
-  while (gtk_events_pending ())  gtk_main_iteration ();
-
-  if (error || !result->image) {
-    if (error) {
-      gsc_trigger_error (context, error);
-    }
-  }
-  else {
-    g_debug ("preview-received");
-    g_signal_emit_by_name (context, "preview-received", result);
-  }
-
-  g_boxed_free (GNOME_TYPE_SCAN_RESULT, result);
+  g_signal_emit_by_name (context, "preview-started", fc); while (gtk_events_pending ())  gtk_main_iteration ();
+  gnome_scan_backend_acquire (priv->backend, info, &error);
+  g_signal_emit_by_name (context, "preview-terminated"); while (gtk_events_pending ())  gtk_main_iteration ();
 }
 
 /**
@@ -823,55 +853,44 @@ gnome_scan_context_forecast (GnomeScanContext *context)
  * @context: a #GnomeScanContext
  * 
  * Acquire a picture. Cancel an acquisition is possible using
- * gnome_scan_context_cancel_acquisition(). The resulted pixbuf is
- * sent using the #GnomeScanContext::"image-received" signal. If the
- * acquisition has been cancel or something else went wrong, only the
- * #GnomeScancontext::"acquisition-terminated" is sent.
+ * gnome_scan_context_stop_acquisition(). The resulted pixbuf is
+ * sent using the #GnomeScanContext::"image-received" signal.
  */
 void
 gnome_scan_context_start_acquisition (GnomeScanContext *context)
 {
-  GnomeScanContextPrivate *priv = GET_PRIVATE (context);
-  GnomeScanResult *result;
-  GdkPixbuf *image;
-  GError *error = NULL;
-
   g_signal_emit_by_name (context, "acquisition-started", gnome_scan_context_forecast (context));
   while (gtk_events_pending ()) gtk_main_iteration ();
-  result = gnome_scan_backend_acquire (priv->backend, priv->info, &error);
-  g_signal_emit_by_name (context, "acquisition-terminated");
-  while (gtk_events_pending ())  gtk_main_iteration ();
-
-  /* If there ware no error (no cancel, etc.) we have a new image. sent it. */
-  if (error || !result->image) {
-    if (error) {
-      gsc_trigger_error (context, error);
-    }
-  }
-  else {
-    /* apply transformations */
-    if (priv->rotation) {
-      image = result->image;
-      result->image = gdk_pixbuf_rotate_simple (image, priv->rotation);
-      g_object_unref (image);
-    }
-
-    g_signal_emit_by_name (context, "image-received", result);
-  }
-  g_boxed_free (GNOME_TYPE_SCAN_RESULT, result);
+  gsc_acquire (context);
 }
 
 /**
- * gnome_scan_context_cancel_acquisition:
+ * gnome_scan_context_resume_acquisition:
+ * @context: a #GnomeScanContext
+ *
+ * Continue an acquisition for multipage document.
+ */
+void
+gnome_scan_context_resume_acquisition (GnomeScanContext *context)
+{
+  g_signal_emit_by_name (context, "acquisition-resumed");
+  gsc_acquire (context);
+}
+
+/**
+ * gnome_scan_context_stop_acquisition:
  * @context: a #GnomeScanContext
  * 
- * Cancel either preview or final acquisition.
+ * Stop an atomic multiple acquisition. If an image is being aquired,
+ * the image acquisition is canceled.
  **/
 void
-gnome_scan_context_cancel_acquisition (GnomeScanContext *context)
+gnome_scan_context_stop_acquisition (GnomeScanContext *context)
 {
   g_return_if_fail (GNOME_IS_SCAN_CONTEXT (context));
-  gnome_scan_backend_cancel (GET_PRIVATE (context)->backend, GET_PRIVATE (context)->info->scanner);
+  gnome_scan_backend_stop (GET_PRIVATE (context)->backend, GET_PRIVATE (context)->info->scanner);
+  g_signal_emit_by_name (context, "acquisition-terminated");
+  while (gtk_events_pending ())  gtk_main_iteration ();
 }
 
 
@@ -890,6 +909,15 @@ gsc_trigger_error (GnomeScanContext *context,
   g_signal_emit_by_name (context, "error", error);
 }
 
+void
+gsc_acquire (GnomeScanContext *context)
+{
+  GnomeScanContextPrivate *priv = GET_PRIVATE (context);
+  GError *error = NULL;
+  gnome_scan_backend_acquire (priv->backend, priv->info, &error);
+  g_signal_emit_by_name (context, "acquisition_paused");
+}
+
 /* This function just propagate data-received backend signal. */
 void
 gsc_data_received (GnomeScanBackend *backend,
@@ -901,3 +929,27 @@ gsc_data_received (GnomeScanBackend *backend,
 
   g_signal_emit_by_name (context, "data-received", data_size);
 }
+
+void
+gsc_image_acquired (GnomeScanBackend *backend,
+		    GnomeScanResult *result,
+		    GnomeScanContext *context)
+{
+  GnomeScanContextPrivate *priv = GET_PRIVATE (context);
+  GdkPixbuf *image;
+
+  if (result->preview) {
+    g_signal_emit_by_name (context, "preview-received", result);
+  }
+  else {
+    /* apply transformations */
+    if (priv->rotation) {
+      result = g_boxed_copy (GNOME_TYPE_SCAN_RESULT, result);
+      image = result->image;
+      result->image = gdk_pixbuf_rotate_simple (image, priv->rotation);
+      g_debug ("%s: Unref image %p", __FUNCTION__, image);
+      g_object_unref (image);
+    }
+    g_signal_emit_by_name (context, "image-received", result);
+  }
+}
